using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.Sql;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace WaterService
{
    public partial class MainForm : Form
    {
        public delegate void WaterSampleDelegate(bool bHighWaterMark, bool bLowWaterMark, int nOverflowLitres, int nTankLitres, Int16 nTemperature, Int16 nMoisture);
        private bool bIsOpen;

        public MainForm()
        {
            bIsOpen = false;
            InitializeComponent();
        }

        private bool IsOpen 
        { 
            get 
            {
                if (!bIsOpen)
                    return false;
                try
                {
                    if ( !serialPort.IsOpen )
                    {
                        System.Diagnostics.Debug.WriteLine("serialPort is not Open");
                        bIsOpen = false;
                        return false;
                    }
                    if (!serialPort.CtsHolding)
                    {
                        System.Diagnostics.Debug.WriteLine("no CTS.  Closing.");
                        serialPort.Close();
                        bIsOpen = false;
                        return false;
                    }
                }
                catch ( Exception  )
                {
                    try
                    {
                        System.Diagnostics.Debug.WriteLine("serialPort exception");
                        serialPort.Close();
                    }
                    catch (Exception )
                    {
                    }
                    bIsOpen = false;
                }
                return bIsOpen; 
            } 
        }

        private void ClosePort()
        {
            try
            {
                if (serialPort.IsOpen)
                    serialPort.Close();
                bIsOpen = false;
            }
            catch (Exception)
            {
            }
        }

        private void OpenPort()
        {
            if (!IsOpen)
            {
                System.Diagnostics.Debug.WriteLine("serialPort Opening");
                try
                {
                    serialPort.Open();
                    bIsOpen = true;
                }
                catch (Exception e)
                {
                    bIsOpen = false;
                    System.Diagnostics.Debug.WriteLine(e.ToString());
                }
            }
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            timer.Tick += new EventHandler(timer_Tick);
            timer.Interval = 15 * 60 * 1000;    // 15 minutes
            serialPort.PortName = "COM7";
            serialPort.DataReceived += new System.IO.Ports.SerialDataReceivedEventHandler(serialPort_DataReceived);

            //OpenPort();
            //if ( IsOpen )
            //    serialPort.WriteLine("\rQuery\r");
            timer_Tick(null, null);
        }

        void timer_Tick(object sender, EventArgs e)
        {
            txtLastPoll.Text = DateTime.Now.ToString();
            chkPollSuccessful.Checked = false;

            try
            {
                if (!IsOpen)
                    OpenPort();

                if (IsOpen)
                {
                    serialPort.WriteLine("\rQuery\r");
                }
            }
            catch (Exception)
            {
                ClosePort();
            }
        }

        void WaterSample(bool bHighWaterMark, bool bLowWaterMark, int nOverflowLitres, int nTankLitres, Int16 nTemperature, Int16 nMoisture)
        {
            System.Data.SqlClient.SqlConnection con=null;
            System.Data.SqlClient.SqlCommand cmd=null;
            try
            {
                con = new System.Data.SqlClient.SqlConnection("Server=MAINPC\\SQLEXPRESS;Integrated Security=true;Initial Catalog=Waterer;");
                con.Open();
                cmd = new System.Data.SqlClient.SqlCommand("usp_AddReading", con);
                cmd.CommandType = CommandType.StoredProcedure;

                System.Data.SqlClient.SqlCommandBuilder.DeriveParameters(cmd);

                cmd.Parameters["@ReadingTime"].Value = DateTime.Now;
                cmd.Parameters["@WaterMarkHigh"].Value = bHighWaterMark;
                cmd.Parameters["@WaterMarkLo"].Value = bLowWaterMark;
                cmd.Parameters["@OverflowLitres"].Value = nOverflowLitres;
                cmd.Parameters["@TankLitres"].Value = nTankLitres;
                cmd.Parameters["@Temperature"].Value = nTemperature;
                cmd.Parameters["@Moisture"].Value = nMoisture;
                cmd.Parameters["@Location"].Value = 1;

                cmd.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine("Exception: " + ex.ToString());
            }
            finally
            {
                if (cmd != null )
                    cmd.Dispose();
                if (con != null)
                {
                    con.Close();
                    con.Dispose();
                }
            }
            chkPollSuccessful.Checked = true;
            chkHighWaterMark.Checked = bHighWaterMark;
            chkLowWaterMark.Checked = bLowWaterMark;
            txtOverflowLitres.Text = ((double)nOverflowLitres / 10.0).ToString();
            txtTankLitres.Text = ((double)nTankLitres / 10.0).ToString();
            txtTemp.Text = ((double)nTemperature / 10.0).ToString();
            txtMoisture.Text = nMoisture.ToString();
        }

        void serialPort_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            if (IsOpen)
            {
                try
                {
                    String s = serialPort.ReadLine();
                    // H,l,-1,87,191
                    String[] args = s.Split(new char[] { ',' });

#if DEBUG
                    {
                        bool _bHighWaterMark = args[0] == "H" ? true : false;
                        bool _bLowWaterMark = args[1] == "L" ? true : false;
                        double _nTankLitres = (Double.Parse(args[2])) / 10.0;
                        double _nOverflowLitres = (Double.Parse(args[3])) / 10.0;
                        double _nTemperature = (Double.Parse(args[4])) / 10.0;

                        System.Diagnostics.Debug.WriteLine("High Water Mark  = " + _bHighWaterMark.ToString());
                        System.Diagnostics.Debug.WriteLine("Low Water Mark   = " + _bLowWaterMark.ToString());
                        System.Diagnostics.Debug.WriteLine("Overflow Litres  = " + _nOverflowLitres.ToString());
                        System.Diagnostics.Debug.WriteLine("Tank Litres      = " + _nTankLitres.ToString());
                        System.Diagnostics.Debug.WriteLine("Temperature      = " + _nTemperature.ToString());
                    }
#endif
                    bool bHighWaterMark = args[0] == "H" ? true : false;
                    bool bLowWaterMark = args[1] == "L" ? true : false;
                    int nTankLitres = Int32.Parse(args[2]);
                    int nOverflowLitres = Int32.Parse(args[3]);
                    Int16 nTemperature = Int16.Parse(args[4]);
                    Int16 nMoisture = 0;

                    this.BeginInvoke( new WaterSampleDelegate(WaterSample), new object[] { bHighWaterMark, bLowWaterMark, nOverflowLitres, nTankLitres, nTemperature, nMoisture} );
                }
                catch (Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine("Exception: " + ex.ToString());
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (IsOpen)
            {
                double d;
                if (!double.TryParse(txtLitres.Text, out d))
                {
                    MessageBox.Show("Not a vaild number");
                }
                else
                {
                    int n = (int)d;
                    serialPort.WriteLine("\rWater " + n.ToString() + "\r");
                }
            }
        }

        private void btnPoll_Click(object sender, EventArgs e)
        {
            timer_Tick(null, null);

        }
    }
}